class item
{
        final static int MODE_INACTIVE = 0;
        final static int MODE_ACTIVE = 1;
        final static int TYPE_BALL = 0;
        final static int TYPE_POWERUP_SPEED = 1;
        final static int TYPE_POWERUP_SLOW = 2;
        final static int TYPE_POWERUP_SUPER = 3;
        final static int TYPE_POWERUP_PAINT = 4;
        final static int NUM_POWERUP_TYPES = 3;
        final static int COLOR_BAD = 10;

        vertex pos, vel;    //position, velocity vectors..
        vertex oldVel;      //stored velocity for later retrieval
        int mode;           //current state
        int type;           //item type
        int color;          //color
        int w, h;           //dimensions
        int points;

        public boolean isInactive()
        {
                return (mode == MODE_INACTIVE);
        }
        public boolean isBad()
        {
                return (color == COLOR_BAD);
        }
        public boolean isStopped()
        {
                return (vel.x == 0 && vel.y == 0);
        }

        //bounce along the x or y axes
        public void bounceHorizontal()
        {
                vel.x *= -1;
        }
        public void bounceVertical()
        {
                vel.y *= -1;
        }

        //stop the item.. (save its current velocity)
        public void stop()
        {
                oldVel.set(vel.x, vel.y, vel.z);
                vel.zero();
        }

        //return to old speed
        public void resume()
        {
                vel.set(oldVel.x, oldVel.y, oldVel.z);
        }

        //motion..
        public void move()
        {
                pos.add(vel);    
        }

        public void kill()
        {
                mode = MODE_INACTIVE;
        }

        //get the center position..
        vertex getCenterPos()
        {
                return new vertex(pos.x+w/2, pos.y+h/2);
        }

        public boolean isPowerup()
        {
                return type >= TYPE_POWERUP_SPEED && type <= TYPE_POWERUP_PAINT;
        }

        //constructors
        public item()
        {
                mode = MODE_INACTIVE;
                pos = new vertex();
                vel = new vertex();
                oldVel = new vertex();
                color = 0;
                type = 0;
                w = h = 16;
                points = 100;
        }
        public item(vertex coords, vertex vCoords, int col, int t, int wid, int hei, int pts)
        {
                mode = MODE_ACTIVE;
                pos = new vertex(coords.x, coords.y, coords.z);
                vel = new vertex(vCoords.x, vCoords.y, vCoords.z);
                oldVel = new vertex(vCoords.x, vCoords.y, vCoords.z);
                color = col;
                type = t;
                w = wid;
                h = hei;
                points = 100;
        }
}
