class player
{
        final static int MODE_INACTIVE = 0;
        final static int MODE_APPEARING = 1;
        final static int MODE_ACTIVE = 2;
        final static int MODE_INVINCIBLE = 3;
        final static int MODE_SLOW = 4;
        final static int MODE_FAST = 5;
        final static int MODE_PAINT = 6;
        final static int MODE_DEAD = 7;
        final static float DEFAULT_ROTATE_SPEED = (float)Math.PI/16.0f;
        final static float DEFAULT_ACCEL_SPEED = 0.05f;
        final static float DEFAULT_DECCEL_SPEED = 0.075f;
        final static float DEFAULT_WALK_SPEED = 1.0f;
        final static int DEFAULT_EYE_SIZE = 3;
        final static float DEFAULT_ROPE_LENGTH = 25.0f;
        final static float DEFAULT_ROPE_SPEED = 6.0f;
        final static int DEFAULT_COMBO_TIME = 10;
        final static int DEFAULT_SPEED_PARTICLE_TIME = 25;
        final static long EXTRA_CREDIT_SCORE = 20000;
        final static int MAX_CREDITS = 6;

        String name = null;       //current name associated with this player
        int mode;                 //current state
        vertex pos,vel;           //position, velocity vectors
        vertex[] eyePos = new vertex[2]; //eye positions
        float angle;              //current rotation angle
        float gunAngle;           //aiming reticile angle
        float rotSpeed;           //rotation speed
        float accel;              //acceleration rate
        float deccel;             //decceleration rate
        float walkSpeed;          //walking speed
        int color;                //current color
        int w, h;                 //body dimensions
        int eyeW, eyeH;           //eye size..
        int dir;                  //current direction (0-4)
        int ropeActive;           //rope active mode
        float ropeSpeed;          //speed of rope deploy
        vertex ropePos, ropeVel;  //rope head position and velocity
        float ropeLength;         //length of rope tail
        float ropeAngle;
        int credits = 3;          //# lives left
        long score = 0;           //current score
        int curNumItems = 0;      //current # items absorbed so far
        clock powerClock;         //powerup timer
        int powerUpDuration;      //duration of current powerup
        int curStreak = 0;        //# of consecutive grabs
        int bestStreak = 0;       //current best streak
        int comboTimer = 0;       //timer for determining combo-age
        int particleTimer = 0;    //timer for spawning particles during boost


        //double the speed
        public void setDoubleSpeed()
        {
                if(vel.x < 0) vel.x = -2*DEFAULT_WALK_SPEED;
                else if(vel.x > 0) vel.x = 2*DEFAULT_WALK_SPEED;
                else if(vel.y < 0) vel.y = -2*DEFAULT_WALK_SPEED;
                else if(vel.y > 0) vel.y = 2*DEFAULT_WALK_SPEED;
                walkSpeed = 2*DEFAULT_WALK_SPEED;
        }
        //back to normal pace..
        public void setNormalSpeed()
        {
                if(vel.x < 0) vel.x = -DEFAULT_WALK_SPEED;
                else if(vel.x > 0) vel.x = DEFAULT_WALK_SPEED;
                else if(vel.y < 0) vel.y = -DEFAULT_WALK_SPEED;
                else if(vel.y > 0) vel.y = DEFAULT_WALK_SPEED;
                walkSpeed = DEFAULT_WALK_SPEED;
        }
        //set to slow-down mode
        public void setSlow(int secs)
        {
                powerUpDuration = secs;
                powerClock = new clock();
                mode = MODE_SLOW;
        }

        //acquire the powerup, and set the current power
        public int acquirePowerupItem(item it)
        {
                it.kill();
                addPoints(it.points+(2*curStreak/3)*4);

                switch(it.type){
                        case item.TYPE_POWERUP_SPEED:
                                powerUpDuration = 8;
                                powerClock = new clock();
                                setDoubleSpeed();
                                particleTimer = 0;
                                mode = MODE_FAST;
                                if(isSlow()) return 5;

                                return 0;
                        case item.TYPE_POWERUP_SLOW:
                                powerUpDuration = 8;
                                powerClock = new clock();
                                if(isSlow()) return -1;
                                mode = MODE_SLOW;
                                setNormalSpeed();

                                return 1;
                        case item.TYPE_POWERUP_SUPER:
                                return 3;
                        case item.TYPE_POWERUP_PAINT:
                                return 4;
                        default:
                                powerUpDuration = 2;
                                powerClock = new clock();
                                break;
                        }
                return -1;
        }

        //increase the score by this point value
        public void addPoints(long points)
        {
                score += points;
        }

        //absorb this item
        public void absorbItem(item it)
        {
                it.kill();
                addPoints(it.points+(2*curStreak/3)*4);
                curNumItems++;
                if(++curStreak > bestStreak){
                        bestStreak = curStreak;
                        }
        }

        //two fb's collide.. this is the reaction..
        public void itemCollisionReaction(item it)
        {
                //normal vector between the two fb's
                vertex n = new vertex(getCenterPos().x-it.getCenterPos().x, getCenterPos().y-it.getCenterPos().y, 0.0f);
                
                float magnitude = (float)Math.sqrt(n.x*n.x+n.y*n.y);

                //normalize the vector (divide by its magnitude)
                n.x /= magnitude;
                n.y /= magnitude;

                //find the length of each movement vector along the normal
                float a1 = vel.x*n.x + vel.y*n.y;
                float a2 = it.vel.x*n.x + it.vel.y*n.y;
                
                //find the potential energy..
                float tempMass = 1000;
                float p = (2.0f*(a1-a2))/(1.0f+tempMass);

                //calculate the new velocity for fba
                vertex va = new vertex(vel.x-p*tempMass*n.x, vel.y-p*tempMass*n.y, 0.0f);

                //calculate the new velocity for fbb
                vertex vb = new vertex(it.vel.x+p*1.0f*n.x, it.vel.y+p*1.0f*n.y, 0.0f);

                //set the velocities..
                vel.x = va.x;
                vel.y = va.y;
                it.vel.x = vb.x;
                it.vel.y = vb.y;

                //make sure the bodies aren't touching by adding the velocities before returning..
                move();
                it.move();
        }

        //get hit by this item
        public int getHitByItem(item it)
        {
                if(!isAppearing()){
                        //change to item's color
                        if(!it.isBad()) color = it.color;
                        //stop..
                        stop();
                
                        curStreak = 0;
                        credits--;
                        if(credits >= 0){
                                powerUpDuration = 2;
                                powerClock = new clock();
                                if(isSlow()){
                                        kill();
                                        return 2;
                                        }
                                kill();
                                return 0;
                                }
                        else{
                                kill();
                                return 1;
                                }
                        }
                else{
                        itemCollisionReaction(it);                        
                        return 0;
                        }
        }

        //kill him!
        public void kill()
        {
                mode = MODE_DEAD;
                powerClock = new clock();
        }

        //left rotation
        public void turnLeft()
        {
                angle -= rotSpeed;
                if(angle < 0) angle += 2.0f*(float)Math.PI;
                eyePos[0].rotate(-rotSpeed);
                eyePos[1].rotate(-rotSpeed);
        }
        //right rotation
        public void turnRight()
        {
                angle += rotSpeed;
                if(angle > 2.0f*(float)Math.PI) angle -= 2.0f*(float)Math.PI;
                eyePos[0].rotate(rotSpeed);
                eyePos[1].rotate(rotSpeed);
        }
        //accelerate
        public void goForward()
        {
                vel.x += accel*(float)Math.cos(angle);
                vel.y += accel*(float)Math.sin(angle);
        }
        //deccelerate
        public void slowDown()
        {
        }
        public void goLeft()
        {
                vel = new vertex(-walkSpeed, 0);
                eyePos[0] = new vertex(-7, -1);
                eyePos[1] = new vertex(-1, -1);
        }
        public void goRight()
        {
                vel = new vertex(walkSpeed, 0);
                eyePos[0] = new vertex(-1, -1);
                eyePos[1] = new vertex(5, -1);
        }                                 
        public void goUp()
        {
                vel = new vertex(0, -walkSpeed);
                eyePos[0] = new vertex(-4, -4);
                eyePos[1] = new vertex(2, -4);
        }
        public void goDown()
        {
                vel = new vertex(0, walkSpeed);
                eyePos[0] = new vertex(-4, 2);
                eyePos[1] = new vertex(2, 2);
        }
        public void stop()
        {
                vel = new vertex();
                eyePos[0] = new vertex(-4, -1);
                eyePos[1] = new vertex(2, -1);
        }
        //fire the rope toward this position
        public void fireRope(int x, int y)
        {
                ropePos = new vertex(pos.x+w/2, pos.y+h/2);
                float ang = (float)Math.atan2(y-ropePos.y, x-ropePos.x);
                ropeVel = new vertex(ropeSpeed*(float)Math.cos(ang), ropeSpeed*(float)Math.sin(ang));
                ropeActive = 1;
                ropeAngle = ang;
        }

        public void setDir(int d)
        {
                dir = d;
        }

        //motion..
        public void move()
        {
                pos.add(vel);
        }
        
        //rope motion..
        public void moveRope()
        {
                ropePos.add(ropeVel);
        }
        
        //count the powerup timer down.. return nonzero to signify 0 was reached
        int handlePowerUpTimer()
        {
                int result = 0;
                powerClock.update();
                if(powerClock.secs >= powerUpDuration){
                        powerClock.secs = -1;
                        switch(mode){
                                case MODE_FAST:
                                        mode = MODE_ACTIVE;
                                        setNormalSpeed();
                                        return 0;

                                case MODE_SLOW:
                                        mode = MODE_ACTIVE;
                                        return 1;

                                case MODE_DEAD:
                                        mode = MODE_INACTIVE;
                                        return 2;

                                case MODE_APPEARING:
                                        mode = MODE_ACTIVE;
                                        goDown();
                                        return 3;

                                default:
                                        mode = MODE_ACTIVE;
                                        return -1;

                                }
                        }
                if(isFast()){
                        if(++particleTimer >= DEFAULT_SPEED_PARTICLE_TIME){
                                particleTimer = 0;
                                return 4;
                                }
                        }
                return -1;
        }

        //get the inverted time remaining..
        int getPowerTimeRemaining()
        {
                return (powerUpDuration-powerClock.secs);
        }

        boolean powerupTimerActive()
        {
                return (mode == MODE_INVINCIBLE || mode == MODE_DEAD || mode == MODE_SLOW || mode == MODE_PAINT || mode == MODE_FAST); // || mode == MODE_APPEARING);
        }

        //tells if the timer has ticked to a new second
        boolean powerupTimerTicked()
        {
                return powerClock.tick;
        }

        //untick the powerup timer
        void unTickPowerupTimer()
        {
                powerClock.unTick();
        }

        boolean isInvincible()
        {
                return mode == MODE_INVINCIBLE;
        }
        boolean isDead()
        {
                return mode == MODE_DEAD;
        }
        boolean isInactive()
        {
                return mode == MODE_INACTIVE;
        }
        boolean isAppearing()
        {
                return mode == MODE_APPEARING;
        }
        boolean isFast()
        {
                return mode == MODE_FAST;
        }
        boolean isSlow()
        {
                return mode == MODE_SLOW;
        }

        //set to invincible mode
        void setInvincible(int sec)
        {
                mode = MODE_INVINCIBLE;
                powerUpDuration = sec;
                powerClock = new clock();
        }
        
        //set to "appear" mode
        void setAppearing()
        {
                mode = MODE_APPEARING;
                powerUpDuration = 3;
                powerClock = new clock();
        }

        //set to normal mode
        void setActive()
        {
                mode = MODE_ACTIVE;
                powerClock.secs = -1;
        }

        //get the center position..
        vertex getCenterPos()
        {
                return new vertex(pos.x+w/2, pos.y+h/2);
        }

        //add a credit
        public void addCredit()
        {
                if(credits < MAX_CREDITS) credits++;
        }

        //set some default values
        void setDefaults()
        {
                mode = MODE_ACTIVE;
                angle = (float)Math.PI/2;
                gunAngle = 0;
                color = 0;
                name = "Player 1";
                w = h = 16;
                rotSpeed = DEFAULT_ROTATE_SPEED;
                accel = DEFAULT_ACCEL_SPEED;
                deccel = DEFAULT_DECCEL_SPEED;
                walkSpeed = DEFAULT_WALK_SPEED;
                eyeW = eyeH = DEFAULT_EYE_SIZE;
                dir = 0;
                eyePos[0] = new vertex(-4, -1);
                eyePos[1] = new vertex(2, -1);
                ropeActive = 0;
                ropeSpeed = DEFAULT_ROPE_SPEED;
                ropeLength = DEFAULT_ROPE_LENGTH;
                ropeAngle = 0;
                ropePos = new vertex();
                ropeVel = new vertex();
                curNumItems = 0;
                powerUpDuration = 0;
                comboTimer = 0;
                particleTimer = 0;
                curStreak = bestStreak = 0;
        }

        //constructors..
        public player()
        {
                setDefaults();
                pos = new vertex();
                vel = new vertex();
        }
        public player(vertex coords, vertex vCoords, int col, int wid, int hei, String str)
        {
                setDefaults();
                //setAppearing();
                color = col;
                name = str;
                pos = new vertex(coords.x, coords.y, coords.z);
                vel = new vertex(vCoords.x, vCoords.y, vCoords.z);
                w = wid;
                h = hei;
        }
}
